"""GL Deep Research Python client library for interacting with the GL DeepResearch API."""

from gl_odr.client import DeepResearchClient
from gl_odr.health import Health
from gl_odr.models import (
    ApiToolOverride,
    HealthResponse,
    McpToolOverride,
    ProfileResponse,
    StreamEvent,
    StreamEventType,
    TaskCreateResponse,
    TaskData,
    TaskgroupAddTaskResponse,
    TaskgroupCreateResponse,
    TaskgroupResponse,
    TaskgroupStatus,
    TaskgroupWebhook,
    TaskResponse,
    TaskStatus,
    TaskStatusResponse,
    WebhookConfig,
)
from gl_odr.profiles import Profiles
from gl_odr.taskgroups import Taskgroups
from gl_odr.tasks import Tasks

__all__ = [
    # Client
    "DeepResearchClient",
    # API Classes
    "Health",
    "Tasks",
    "Taskgroups",
    "Profiles",
    # Enums
    "TaskStatus",
    "TaskgroupStatus",
    "StreamEventType",
    # Health Models
    "HealthResponse",
    "StreamEvent",
    # Task Models
    "WebhookConfig",
    "ApiToolOverride",
    "McpToolOverride",
    "TaskCreateResponse",
    "TaskData",
    "TaskResponse",
    "TaskStatusResponse",
    # Taskgroup Models
    "TaskgroupWebhook",
    "TaskgroupCreateResponse",
    "TaskgroupResponse",
    "TaskgroupAddTaskResponse",
    # Profile Models
    "ProfileResponse",
]
